"""
Copyright (c) 2024 Baidu.com, Inc. All Rights Reserved
This module provide configigure file management service in i18n environment.

Authors: zhangjiabin01
Date: 2024/5/9 10:26:00
"""
import configparser
import json


def replace_values(json_obj, replace_dict):
    """
    替换json中的内容，替换
    :param json_obj:
    :param replace_dict:
    :return:
    """
    if isinstance(json_obj, dict):
        return {k: replace_values(v, replace_dict) for k, v in json_obj.items()}
    elif isinstance(json_obj, list):
        return [replace_values(element, replace_dict) for element in json_obj]
    else:
        return replace_dict.get(json_obj, json_obj)


def save_conf_var(ini_file):
    """
    根据ini文件，生成一个dict，用于替换json中的变量
    :param ini_file:
    :return:
    """
    config = configparser.ConfigParser()
    config.read(ini_file)


def generate_value_to_jsonpath_map(obj, path='$'):
    value_to_path = {}

    if isinstance(obj, dict):
        for key, value in obj.items():
            new_path = f'{path}.{key}' if path != '$' else f'$.{key}'
            if isinstance(value, (dict, list)):
                nested_paths = generate_value_to_jsonpath_map(value, new_path)
                for val, paths in nested_paths.items():
                    if val not in value_to_path:
                        value_to_path[val] = []
                    if isinstance(paths, list):
                        value_to_path[val].extend(paths)
                    else:
                        value_to_path[val].append(paths)
            else:
                if value not in value_to_path:
                    value_to_path[value] = []
                value_to_path[value].append(new_path)

    elif isinstance(obj, list):
        for index, value in enumerate(obj):
            new_path = f'{path}[{index}]'
            if isinstance(value, (dict, list)):
                nested_paths = generate_value_to_jsonpath_map(value, new_path)
                for val, paths in nested_paths.items():
                    if val not in value_to_path:
                        value_to_path[val] = []
                    if isinstance(paths, list):
                        value_to_path[val].extend(paths)
                    else:
                        value_to_path[val].append(paths)
            else:
                if value not in value_to_path:
                    value_to_path[value] = []
                value_to_path[value].append(new_path)

    return value_to_path


if __name__ == '__main__':
    original_dict = [{"url": "https://aiob-open.baidu.com", "path": "/aiob-server/api/v2/getToken", "method": "POST", "headers": {"Content-Type": "application/json"}, "params": {}, "body": {"accessKey": "0600332f717442ae9198e3f39f22594b", "secretKey": "ed81a69cffb849c98f587a4b367a5151"}}, {"url": "https://aiob-open.baidu.com", "path": "/api/v1/did/list", "method": "GET", "headers": {"Authorization": "cc-api-auth-v2/10.D392A0628FDD362CED972BA925DC20D6C08796CA6D5D7FE89465FE3DA050A3EFE89A08C20A05204E939AA902D55BA058"}, "params": {}, "body": {}}]

    new_dict = generate_value_to_jsonpath_map(original_dict)
    print(json.dumps(new_dict))



"""
Copyright (c) 2024 Baidu.com, Inc. All Rights Reserved
This module provide configigure file management service in i18n environment.

Authors: zhangjiabin01
Date: 2024/5/8 16:56:00
"""
import json
from urllib.parse import urlparse, parse_qs


def analysis(har_file_path, res_file_path=None):
    """
    解析har文件，生成测试用例的json格式
    :param har_file_path: har文件路径
    :param res_file_path: 解析结果文件路径，如果为None，返回json格式字符串
    """
    with open(har_file_path, 'r', encoding='utf-8') as f:
        har_json = f.read()
    analysis_json(har_json, res_file_path)


def analysis_json(har_json, res_file_path=None):
    """
    解析har文件，生成测试用例的json格式
    :param har_json:
    :param res_file_path:
    :return:
    """
    har_dict = json.loads(har_json)
    json_output = {
        "caseName": "",
        "description": "",
        "priority": 0,
        "testSteps": []
    }

    # 提取 request 信息
    for index, entry in enumerate(har_dict['log']['entries']):
        request = entry['request']
        parsed_url = urlparse(request['url'])
        url = parsed_url.scheme + '://' + parsed_url.netloc
        path = parsed_url.path
        params = {k: v[0] for k, v in parse_qs(parsed_url.query).items()}
        body = request.get('postData', {})
        if 'json' in body.get('mimeType', ''):  # 检查MIME类型是否为JSON
            body = json.loads(body.get('text', '{}'))  # 是则解析
        step = {
            "step": index,
            "request": {
                "url": url,
                "path": path,
                "method": request['method'],
                "headers": {header['name']: header['value'] for header in request['headers']},
                "param": params,
                "body": body
            },
            "action": [],
            "assert": [
                {
                    "expect": "",
                    "actual": "",
                    "operator": ""
                }
            ]
        }
        json_output['testSteps'].append(step)

    # 输出结果
    res_str = json.dumps(json_output, indent=4)
    if res_file_path:
        with open(res_file_path, 'w', encoding='utf-8') as f:
            f.write(res_str)
        return res_str
    else:
        return res_str


if __name__ == '__main__':
    analysis('/Users/zhangjiabin01/Desktop/new_auto/baidu/kefu-qa/bintest/har/通知机器人文本测试.har',
             '/Users/zhangjiabin01/Desktop/new_auto/baidu/kefu-qa/bintest/har/通知机器人文本测试.json')